﻿namespace WebSharperSiteletsProject

open System
open System.IO
open System.Web
open IntelliFactory.WebSharper.Sitelets

/// The website definition.
module Html5ClockSite =
    open IntelliFactory.Html
    open IntelliFactory.WebSharper

    /// Actions that corresponds to the different pages in the site.
    type Action =
        | Home
        | Clock
        | Login of option<Action>
        | Logout

    /// A helper function to create a hyperlink
    let private ( => ) title href =
        A [HRef href] -< [Text title]

    /// A helper function to create a 'fresh' url with a random get parameter
    /// in order to make sure that browsers don't show a cached version.
    let private RandomizeUrl url =
        url + "?d=" +
            System.Uri.EscapeUriString (System.DateTime.Now.ToString())

    /// User-defined widgets.
    module Widgets =

        /// Widget for displaying login status or a link to login.
        let LoginInfo (ctx: Context<Action>) =
            let user = UserSession.GetLoggedInUser ()
            [
                (
                    match user with
                    | Some email ->
                        "Log Out (" + email + ")" => 
                            (RandomizeUrl <| ctx.Link Action.Logout)
                    | None ->
                        "Login" => (ctx.Link <| Action.Login None)
                )
            ]

    /// A template function that renders a page with some customizations.
    let Template title main : Content<Action> =
        let menu (ctx: Context<Action>)=
            let ( ! ) x = ctx.Link x
            [
                    "Home"      => !Action.Home
                    "Clock"     => (RandomizeUrl <| !Action.Clock)
                    "ASPX Page" => "/LegacyPage.aspx"
            ]
            |> List.map (fun link -> 
                Label [Class "menu-item"] -< [link]
            )
        Templates.Skin.Skin (Some title)
            {
                LoginInfo   = Widgets.LoginInfo
                Banner      = fun ctx -> [H2 [Text title]]
                Menu        = menu
                Main        = main
                Sidebar     = fun ctx -> [Text "Put your side bar here"]
                Footer      = fun ctx -> [Text "Your website.  Copyright (c) 2011 YourCompany.com"]
            }

    /// The pages of this website.
    module Pages =

        /// The home page.
        let HomePage : Content<Action> =
            Template "Home" <| fun ctx ->
                [
                    H1 [Text "Welcome to our site!"]
                ]

        /// A simple login page.
        let LoginPage (redirectAction: option<Action>): Content<Action> =
            Template "Login" <| fun ctx ->
                let redirectLink =
                    match redirectAction with
                    | Some action -> action
                    | None        -> Action.Home
                    |> ctx.Link
                [
                    H1 [Text "Login"]
                    P [
                        Text "Login with any username and password='"
                        I [Text "password"]
                        Text "'."
                    ]
                    Div [
                        new LoginControl(redirectLink)
                    ]
                ]

        /// A simple page that shows a clock.
        let ClockPage : Content<Action> =
            Template "Clock" <| fun ctx ->
                [
                    H1 [Text "Drawing in HTML5 is easy!"]
                    Div [
                        new ClockControl()
                    ]
                ]

    /// The sitelet that corresponds to the entire site.
    let EntireSite =
        // A simple sitelet for the home page, available at the root of the application.
        let home = 
            Sitelet.Content "/" Action.Home Pages.HomePage

        // An automatically inferred sitelet created for the basic parts of the application.
        let basic =
            Sitelet.Infer <| fun action ->
                match action with
                | Action.Home ->
                    Pages.HomePage
                | Action.Login action->
                    Pages.LoginPage action
                | Action.Logout ->
                    // Logout user and redirect to home
                    UserSession.Logout ()
                    Content.Redirect Action.Home
                | Action.Clock ->
                    Content.ServerError

        // A sitelet for the protected content that requires users to log in first.
        let authenticated =
            let filter : Sitelet.Filter<Action> =
                {
                    VerifyUser = fun _ -> true
                    LoginRedirect = Some >> Action.Login
                }

            Sitelet.Protect filter <|
                Sitelet.Content "/protected" Action.Clock Pages.ClockPage

        // Compose the above sitelets into a larger one.
        [
            home
            authenticated
            basic
        ]
        |> Sitelet.Sum

/// Expose the main sitelet so that it can be served.
/// This needs an IWebsite type and an assembly level annotation.
type Html5ClockWebsite() =
    interface IWebsite<Html5ClockSite.Action> with
        member this.Sitelet = Html5ClockSite.EntireSite
        member this.Actions = []

[<assembly: WebsiteAttribute(typeof<Html5ClockWebsite>)>]
do ()
